/*  The Blue Mango Quest
 *  Copyright (c) Clment 'phneutre' Bourdarias (code)
 *                   email: phneutre@users.sourceforge.net
 *                Guillaume 'GuBuG' Burlet (graphics)
 *                   email: gubug@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "timers.h"
#include "world_building.h"
#include "world_geometry.h"
#include "hut.h"
#include "bonus.h"
#include "mango.h"
#include "map.h"

extern game_data_t *world;

void load_map(char *filename)
{
  gzFile map_file=0;
  int compatibility_n=0;

  char *verification=0;

  printf("Verifying map... ");

  if (!(map_file = gzopen(filename, "rb9")))
  {
    fprintf(stderr, "\nError opening file %s.\n", filename);
    exit(1);
  }

  verification = new char[3];

  gzread(map_file, verification, sizeof(char)*3);

  if ((verification[0] != 'S') || (verification[1] != 'H')
      || (verification[2] != 'X'))
  {
    printf("\nError: file is not a map for MangoQuest\n");
    delete[]verification;
    exit(1);
  }
  delete[]verification;

  gzread(map_file,&compatibility_n, sizeof(int)*1);

  if (compatibility_n != COMPAT_NUMBER)
  {
    printf("\nError: This map is not compatible with current version.\n");
    exit(1);
  }
  printf("this seems to be good!\n");

  load_description(map_file);
  load_geometry(map_file);

  gzclose(map_file);
  map_file = 0;

}

void load_description(gzFile the_file)
{
  int j = 0, k = 0, i = 0;
  int *temp_data=0;

  printf("Loading header and grid data... ");
  gzread(the_file,&world->size, sizeof(int)*1);
  gzread(the_file,&world->country, sizeof(int)*1);
  gzread(the_file,&world->time_limit, sizeof(int)*1);
  gzread(the_file,&world->num_huts, sizeof(int)*1);

  if (world->num_huts == 0)
  {
    fprintf(stderr, "\nFatal error : there is no Shmollux in here !\n");
    exit(1);
  }
  world->size_square = world->size * world->size;
  world->hut_squares = new int[world->num_huts];

  temp_data = new int[world->size_square * 7];
  gzread(the_file, temp_data, sizeof(int)* world->size_square * 7);

  world->map = new SQUARE[world->size_square];

  k = 0;
  for (j = 0; j < world->size_square; j++)
  {

    world->map[j].tex_sol = temp_data[k];
    world->map[j].tex_pla = temp_data[k + 1];
    world->map[j].tex_mur = temp_data[k + 2];
    world->map[j].bonus = temp_data[k + 3];
    world->map[j].paramBonus = temp_data[k + 4];
    world->map[j].special = temp_data[k + 5];
    world->map[j].paramSpecial = temp_data[k + 6];

    if ((world->map[j].special > -1)
        && (world->map[j].special <= SHMIXMAN_B))
    {
      world->start_square = j;
    }

    if (world->map[j].special == FOYER)
    {
      if (i > world->num_huts)
      {
        fprintf(stderr, "\nFatal error : bad map.\n");
        exit(1);
      }
      world->hut_squares[i] = j;
      i++;
    }

    k += 7;
  }
  delete[]temp_data;
  printf("done!\n");

}

void load_geometry(gzFile the_file)
{
  int i=0;
  printf("Loading geometry data... ");

  gzread(the_file, &world->total_num_portals, sizeof(int)*1);
  register_portals(world->total_num_portals);
  read_all_portals(the_file);
  
  gzread(the_file, &world->num_sectors, sizeof(int)*1);
  register_sectors(world->num_sectors);

  for (i=0; i < world->num_sectors; i++)
    {

      gzread(the_file, &world->sectors[i].xmin, sizeof(double)*1);
      gzread(the_file, &world->sectors[i].zmin, sizeof(double)*1);
      gzread(the_file, &world->sectors[i].xmax, sizeof(double)*1);
      gzread(the_file, &world->sectors[i].zmax, sizeof(double)*1);

      gzread(the_file, &world->sectors[i].num_portals, sizeof(int)*1);
      register_portals_indices(i);
      read_portals_numbers(the_file, i);

      gzread(the_file, &world->sectors[i].num_triangles, sizeof(int)*1);
      register_triangles(i);
      read_triangles(the_file, i);
      

      gzread(the_file, &world->sectors[i].num_bonus, sizeof(int)*1);
      register_bonus(i);
      read_bonus(the_file, i);

    }

  printf("done!\n");
  printf("Map has %d sectors.\n", world->num_sectors);
}

void read_all_portals(gzFile the_file)
{
  for (int j=0; j < world->total_num_portals; j++)
    {
      gzread(the_file, &world->portals[j].x1, sizeof(double)*1);
      gzread(the_file, &world->portals[j].z1, sizeof(double)*1);
      gzread(the_file, &world->portals[j].x2, sizeof(double)*1);
      gzread(the_file, &world->portals[j].z2, sizeof(double)*1);
      gzread(the_file, &world->portals[j].TYPE, sizeof(char)*1);
      gzread(the_file, &world->portals[j].number, sizeof(int)*1);
    }

}

void read_portals_numbers(gzFile the_file, int indice)
{
  for (int j=0; j < world->sectors[indice].num_portals; j++)
    {
      gzread(the_file, &world->sectors[indice].portal_indices[j], sizeof(int)*1);
    }

}


void read_triangles(gzFile the_file, int indice)
{
  for (int j=0; j < world->sectors[indice].num_triangles; j++)
    {

      for (int k=0; k < 3; k++)
	{
	  gzread(the_file, &world->sectors[indice].triangles[j].vertex[k].x, 
		sizeof(double)*1);
	  gzread(the_file, &world->sectors[indice].triangles[j].vertex[k].y, 
		sizeof(double)*1);
	  gzread(the_file, &world->sectors[indice].triangles[j].vertex[k].z, 
		sizeof(double)*1);
	  gzread(the_file, &world->sectors[indice].triangles[j].vertex[k].u, 
		sizeof(double)*1);
	  gzread(the_file, &world->sectors[indice].triangles[j].vertex[k].v, 
		sizeof(double)*1);
	}
      gzread(the_file, &world->sectors[indice].triangles[j].normal_x, 
	    sizeof(double)*1);
      gzread(the_file, &world->sectors[indice].triangles[j].normal_y, 
	    sizeof(double)*1);
      gzread(the_file, &world->sectors[indice].triangles[j].normal_z, 
	    sizeof(double)*1);
      gzread(the_file, &world->sectors[indice].triangles[j].texture, 
	    sizeof(int)*1);
      gzread(the_file, &world->sectors[indice].triangles[j].type, 
	    sizeof(char)*1);
    }
}

void read_bonus(gzFile the_file, int indice)
{
  for (int j=0; j < world->sectors[indice].num_bonus; j++)
    {
      gzread(the_file, &world->sectors[indice].bonus[j].square,sizeof(int)*1);
      gzread(the_file, &world->sectors[indice].bonus[j].x,sizeof(double)*1);
      gzread(the_file, &world->sectors[indice].bonus[j].y,sizeof(double)*1);
      gzread(the_file, &world->sectors[indice].bonus[j].z,sizeof(double)*1);

      gzread(the_file, &world->sectors[indice].bonus[j].type,sizeof(int)*1);
      gzread(the_file, &world->sectors[indice].bonus[j].parameter,sizeof(int)*1);
    }

}
